import React from 'react';
import './index.scss';
export declare type StandardColumnItem = {
    value: string;
    label: string;
    children?: ColumnItem[];
};
export declare type ColumnItem = {
    [key: string]: any;
    value?: string;
    label?: string;
    children?: ColumnItem[];
};
export declare type CascaderProps = {
    title?: string;
    height?: number;
    columnWidth?: number;
    maxSelection?: number;
    maxSelectionMsg?: string;
    changeOnSelect?: boolean;
    showToolbar?: boolean;
    toolbarPosition?: 'top' | 'bottom';
    cancelButtonText?: string;
    confirmButtonText?: string;
    valueKey?: string | string[];
    labelKey?: string | string[];
    data?: ColumnItem[];
    defaultValue?: string[] | string[][];
    checkedIcon?: JSX.Element;
    uncheckedIcon?: JSX.Element;
    onLoad?: (value: string[]) => Promise<ColumnItem[]>;
    onCancel?: () => void;
    onConfirm?: (value: string[] | string[][]) => void;
    closePopup?: (confirm?: boolean) => void;
    onChange?: (value: string[] | string[][]) => void;
    onSelectionExceeds?: () => void;
    _popupId?: number;
};
declare type CascaderState = {
    pickerValue: string[][];
    rollbackPickerValue: string[][];
    currentValue: string[];
    rollbackCurrentValue: string[];
    contentWidth: number;
    backSteps: number;
    loading: boolean;
    data?: ColumnItem[];
    prevProps: CascaderProps;
};
export declare class Cascader extends React.PureComponent<CascaderProps, CascaderState> {
    static defaultProps: {
        height: number;
        columnWidth: number;
        maxSelection: number;
        showToolbar: boolean;
        toolbarPosition: string;
        valueKey: string;
        labelKey: string;
        data: ColumnItem[];
    };
    private contentRef;
    constructor(props: CascaderProps);
    static getDerivedStateFromProps(props: CascaderProps, state: CascaderState): CascaderState;
    componentDidMount(): void;
    componentWillUnmount(): void;
    onPopupOpened(id: number): void;
    getValue(): string[] | string[][];
    clearValue(cb: () => void): void;
    showLoading(): void;
    hideLoading(): void;
    genToolbar(top?: boolean): JSX.Element;
    onConfirm(): void;
    onCancel(): void;
    normalizeItem(item: ColumnItem): StandardColumnItem;
    isSameValue(a: string[], b: string[]): boolean;
    isContainValue(a: string[], b: string[]): boolean;
    isItemSelected(columnIndex: number, item: StandardColumnItem): boolean;
    hasChildrenSelected(columnIndex: number, item: StandardColumnItem): boolean;
    onClickItem(columnIndex: number, item: StandardColumnItem): void;
    getColumnWidth(): number;
    getOffset(columns: number): number;
    getColumnItems(values: string[]): ColumnItem[];
    genColumns(): JSX.Element[];
    backOneStep(): void;
    forwardOneStep(): void;
    genBackBtn(): JSX.Element;
    genForwardBtn(): JSX.Element;
    render(): JSX.Element;
}
export {};
