var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { jsx as _jsx } from "react/jsx-runtime";
import React from 'react';
import { get } from 'lodash-es';
import { Cell } from '../cell';
import { CellGroup } from '../cell-group';
import { Checkbox } from '../checkbox';
import { omit, isDef } from '../utils';
import { createBEM } from '../utils/bem';
import './index.scss';
var bem = createBEM('pant-checkbox-group');
var CheckboxGroup = /** @class */ (function (_super) {
    __extends(CheckboxGroup, _super);
    function CheckboxGroup() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.bindedOnClick = _this.onClick.bind(_this);
        _this.state = {
            value: _this.props.defaultValue,
        };
        return _this;
    }
    CheckboxGroup.prototype.getValue = function () {
        return this.state.value;
    };
    CheckboxGroup.prototype.toggleAll = function (on) {
        var _this = this;
        var oldValue = this.state.value;
        var options = this.props.options;
        var newValue;
        if (on) {
            newValue = options.map(function (option) {
                return _this.normalizeOption(option).value;
            });
            if (newValue.every(function (value) {
                return oldValue.includes(value);
            })) {
                return;
            }
        }
        else {
            if (!oldValue.length) {
                return;
            }
            newValue = [];
        }
        this.setNewValue(newValue);
    };
    CheckboxGroup.prototype.setNewValue = function (value) {
        var props = this.props;
        var onChange = props.onChange;
        this.setState({ value: value }, function () {
            onChange && onChange(value, props);
        });
    };
    CheckboxGroup.prototype.normalizeOption = function (option) {
        var _a = this.props, valueKey = _a.valueKey, labelKey = _a.labelKey;
        if (typeof option === 'string') {
            return { label: option, value: option };
        }
        var value = get(option, valueKey, '') + '';
        return __assign(__assign({}, option), { label: get(option, labelKey, value), value: value });
    };
    CheckboxGroup.prototype.onClick = function (event, checkboxProps) {
        var props = this.props;
        var max = props.max, onMaxLimit = props.onMaxLimit, onClick = props.onClick;
        var oldValue = this.state.value;
        var newValue;
        if (checkboxProps.checked) {
            if (props.role === 'radio') {
                return;
            }
            newValue = oldValue.filter(function (value) {
                return value !== checkboxProps.value;
            });
        }
        else {
            if (max > 0 && +max === oldValue.length) {
                if (max > 1) {
                    onMaxLimit && onMaxLimit(props);
                    onClick && onClick(event, checkboxProps);
                    return;
                }
                newValue = [checkboxProps.value];
            }
            else {
                newValue = [checkboxProps.value].concat(oldValue);
            }
        }
        this.setNewValue(newValue);
        onClick && onClick(event, checkboxProps);
    };
    CheckboxGroup.prototype.onCellClick = function (checkboxProps, event) {
        this.onClick(event, checkboxProps);
    };
    CheckboxGroup.prototype.genOption = function (option, passProps) {
        var props = this.props;
        var stateValue = this.state.value;
        option = this.normalizeOption(option);
        var label = option.label;
        option = __assign(__assign({}, option), { label: '' });
        var checkboxProps = __assign(__assign({}, passProps), { value: option.value, iconNode: props.iconNode, activeIconNode: props.activeIconNode, inactiveIconNode: props.inactiveIconNode, disabled: isDef(option.disabled) ? option.disabled : props.disabled, checked: stateValue.includes(option.value) });
        if (props.cellGroup) {
            return (_jsx(Cell, { title: label, rightIcon: _jsx(Checkbox, __assign({}, checkboxProps, { children: option.label }), void 0), onClick: this.onCellClick.bind(this, checkboxProps) }, option.value));
        }
        return (_jsx(Checkbox, __assign({}, checkboxProps, { onClick: this.bindedOnClick }, { children: label }), option.value));
    };
    CheckboxGroup.prototype.genOptions = function () {
        var _this = this;
        var props = this.props;
        var options = props.options;
        var passProps = omit(props, ['name', 'options', 'defaultValue', 'max', 'onClick', 'onChange', 'onMaxLimit']);
        if (props.cellGroup) {
            return _jsx(CellGroup, __assign({}, props.cellGroup, { children: options.map(function (option) { return _this.genOption(option, passProps); }) }), void 0);
        }
        return options.map(function (option) { return _this.genOption(option, passProps); });
    };
    CheckboxGroup.prototype.render = function () {
        return _jsx("div", __assign({ className: bem([this.props.direction]) }, { children: this.genOptions() }), void 0);
    };
    CheckboxGroup.defaultProps = {
        disabled: false,
        labelPosition: 'right',
        shape: 'round',
        direction: 'vertical',
        options: [],
        valueKey: 'value',
        labelKey: 'label',
        defaultValue: [],
    };
    return CheckboxGroup;
}(React.PureComponent));
export { CheckboxGroup };
