var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import React from 'react';
import { on, off } from '../utils/event';
import { isHidden } from '../utils/dom';
import { createBEM } from '../utils/bem';
import { getScroller } from '../utils/scroll';
import { Loading } from '../loading';
import { i18n } from '../locale';
import './index.scss';
var bem = createBEM('pant-list');
var List = /** @class */ (function (_super) {
    __extends(List, _super);
    function List(props) {
        var _this = _super.call(this, props) || this;
        _this.containerRef = React.createRef();
        _this.placeholderRef = React.createRef();
        _this.state = {
            loading: false,
            loadResult: {},
        };
        _this.check = _this.check.bind(_this);
        _this.onClickError = _this.onClickError.bind(_this);
        return _this;
    }
    List.prototype.componentDidMount = function () {
        this.scroller = getScroller(this.containerRef.current);
        on(this.scroller, 'scroll', this.check);
        this.check();
    };
    List.prototype.componentDidUpdate = function () {
        this.check();
    };
    List.prototype.componentWillUnmount = function () {
        off(this.scroller, 'scroll', this.check);
        this.scroller = null;
    };
    List.prototype.reset = function (callback) {
        this.setState({ loading: false, loadResult: {} }, callback);
    };
    List.prototype.check = function () {
        var _a = this.state, loading = _a.loading, loadResult = _a.loadResult;
        if (loading || loadResult.finished || loadResult.error) {
            return;
        }
        if (isHidden(this.containerRef.current)) {
            return;
        }
        var scroller = this.scroller;
        var _b = this.props, offset = _b.offset, direction = _b.direction;
        var scrollerRect;
        if (scroller.getBoundingClientRect) {
            scrollerRect = scroller.getBoundingClientRect();
        }
        else {
            scrollerRect = {
                top: 0,
                bottom: scroller.innerHeight,
            };
        }
        var scrollerHeight = scrollerRect.bottom - scrollerRect.top;
        if (!scrollerHeight) {
            return;
        }
        var isReachEdge = false;
        var placeholderRect = this.placeholderRef.current.getBoundingClientRect();
        if (direction === 'up') {
            isReachEdge = scrollerRect.top - placeholderRect.top <= offset;
        }
        else {
            isReachEdge = placeholderRect.bottom - scrollerRect.bottom <= offset;
        }
        if (isReachEdge) {
            this.load();
        }
    };
    List.prototype.load = function (error) {
        var _this = this;
        this.setState({ loading: true, loadResult: {} }, function () {
            _this.props.onLoad(error).then(function (res) {
                _this.setState({ loading: false, loadResult: res || {} });
            });
        });
    };
    List.prototype.genLoading = function () {
        var _a = this.state, loading = _a.loading, loadResult = _a.loadResult;
        var _b = this.props, loadingText = _b.loadingText, loadingNode = _b.loadingNode;
        if (loading && !loadResult.finished) {
            return (_jsx("div", __assign({ className: bem('loading') }, { children: loadingNode || _jsx(Loading, __assign({ size: "16" }, { children: loadingText || i18n().loading }), void 0) }), "loading"));
        }
    };
    List.prototype.genFinishedText = function () {
        var loadResult = this.state.loadResult;
        var _a = this.props, finishedText = _a.finishedText, finishedNode = _a.finishedNode;
        if (loadResult.finished) {
            var text = finishedNode || finishedText;
            if (text) {
                return _jsx("div", __assign({ className: bem('finished-text') }, { children: text }), void 0);
            }
        }
    };
    List.prototype.genErrorText = function () {
        var loadResult = this.state.loadResult;
        var _a = this.props, errorText = _a.errorText, errorNode = _a.errorNode;
        if (loadResult.error) {
            var text = loadResult.errorNode || loadResult.errorText || errorNode || errorText;
            if (text) {
                return (_jsx("div", __assign({ onClick: this.onClickError, className: bem('error-text') }, { children: text }), void 0));
            }
        }
    };
    List.prototype.onClickError = function () {
        this.load(true);
    };
    List.prototype.render = function () {
        var _a = this.props, direction = _a.direction, children = _a.children;
        var Placeholder = _jsx("div", { ref: this.placeholderRef, className: bem('placeholder') }, void 0);
        return (_jsxs("div", __assign({ ref: this.containerRef, className: bem(), role: "feed", "aria-busy": this.state.loading }, { children: [direction === 'down' ? children : Placeholder, this.genLoading(), this.genFinishedText(), this.genErrorText(), direction === 'up' ? children : Placeholder] }), void 0));
    };
    List.defaultProps = {
        direction: 'down',
        offset: 300,
    };
    return List;
}(React.PureComponent));
export { List };
