/**
 * bem helper
 * b() // 'button'
 * b('text') // 'button__text'
 * b({ disabled }) // 'button button--disabled'
 * b('text', { disabled }) // 'button__text button__text--disabled'
 * b(['disabled', 'primary']) // 'button button--disabled button--primary'
 */
var ELEMENT = '__';
var MODS = '--';
function join(name, el, symbol) {
    return el ? name + symbol + el : name;
}
function prefix(name, mods) {
    if (typeof mods === 'string') {
        return join(name, mods, MODS);
    }
    if (Array.isArray(mods)) {
        return mods.map(function (item) { return prefix(name, item); }).join(' ');
    }
    var ret = [];
    if (mods) {
        Object.keys(mods).forEach(function (key) {
            if (mods[key]) {
                ret.push(name + MODS + key);
            }
        });
    }
    return ret.join(' ');
}
export function createBEM(name) {
    return function (el, mods) {
        var res;
        if (typeof el === 'string') {
            res = join(name, el, ELEMENT);
        }
        else {
            mods = el;
            res = name;
        }
        if (mods) {
            res = [res, prefix(res, mods)].join(' ').trim();
        }
        return res;
    };
}
