function isWindow(val) {
    return val === window;
}
// get nearest scroll element
// http://w3help.org/zh-cn/causes/SD9013
// http://stackoverflow.com/questions/17016740/onscroll-function-is-not-working-for-chrome
var overflowScrollReg = /scroll|auto/i;
export function getScroller(el, root) {
    if (root === void 0) { root = window; }
    var node = el;
    while (node && node.tagName !== 'HTML' && node.nodeType === 1 && node !== root) {
        var overflowY = getComputedStyle(node).overflowY;
        if (overflowScrollReg.test(overflowY)) {
            if (node.tagName !== 'BODY') {
                return node;
            }
            // see: https://github.com/youzan/vant/issues/3823
            var htmlOverflowY = getComputedStyle(node.parentNode).overflowY;
            if (overflowScrollReg.test(htmlOverflowY)) {
                return node;
            }
        }
        node = node.parentNode;
    }
    return root;
}
export function isRootScroller(el) {
    if (el === window) {
        return true;
    }
    var tagName = el.tagName;
    return tagName === 'BODY' || tagName === 'HTML';
}
export function getScrollTop(el) {
    if (isRootScroller(el)) {
        return getRootScrollTop();
    }
    return 'scrollTop' in el ? el.scrollTop : el.pageYOffset;
}
export function setScrollTop(el, value) {
    if ('scrollTop' in el) {
        el.scrollTop = value;
    }
    else {
        el.scrollTo(el.scrollX, value);
    }
}
export function getRootScrollTop() {
    return window.pageYOffset || document.documentElement.scrollTop || document.body.scrollTop || 0;
}
export function setRootScrollTop(value) {
    setScrollTop(window, value);
    setScrollTop(document.body, value);
}
// get distance from element top to scroller content top
export function getElementTop(el, scroller) {
    if (isWindow(el)) {
        return 0;
    }
    if (!scroller || isRootScroller(scroller)) {
        return el.getBoundingClientRect().top + getRootScrollTop();
    }
    return (el.getBoundingClientRect().top - scroller.getBoundingClientRect().top + getScrollTop(scroller));
}
export function getVisibleHeight(el) {
    if (isWindow(el)) {
        return el.innerHeight;
    }
    return el.getBoundingClientRect().height;
}
export function getVisibleTop(el) {
    if (isWindow(el)) {
        return 0;
    }
    return el.getBoundingClientRect().top;
}
export function getVisibleBottom(el) {
    if (isWindow(el)) {
        return 0;
    }
    return el.getBoundingClientRect().bottom;
}
export function getViewportSize() {
    return {
        width: Math.max(document.documentElement.clientWidth || 0, window.innerWidth || 0),
        height: Math.max(document.documentElement.clientHeight || 0, window.innerHeight || 0),
    };
}
export function scrollLeftTo(scroller, to, duration) {
    var count = 0;
    var from = scroller.scrollLeft;
    var frames = duration === 0 ? 1 : Math.round((duration * 1000) / 16);
    var step = (to - from) / frames;
    function animate() {
        scroller.scrollLeft += step;
        if (++count < frames) {
            requestAnimationFrame(animate);
        }
    }
    animate();
}
export function scrollTopTo(scroller, to, duration, callback) {
    var current = getScrollTop(scroller);
    var isDown = current < to;
    var frames = duration === 0 ? 1 : Math.round((duration * 1000) / 16);
    var step = (to - current) / frames;
    function animate() {
        current += step;
        if ((isDown && current > to) || (!isDown && current < to)) {
            current = to;
        }
        setScrollTop(scroller, current);
        if ((isDown && current < to) || (!isDown && current > to)) {
            requestAnimationFrame(animate);
        }
        else if (callback) {
            requestAnimationFrame(callback);
        }
    }
    animate();
}
