var MIN_DISTANCE = 10;
var TouchHandler = /** @class */ (function () {
    function TouchHandler(el, opt) {
        this.startX = 0;
        this.startY = 0;
        this.deltaX = 0;
        this.deltaY = 0;
        this.offsetX = 0;
        this.offsetY = 0;
        this.el = el;
        this.opt = opt || {};
        this.onTouchStart = this.onTouchStart.bind(this);
        this.onTouchMove = this.onTouchMove.bind(this);
        this.onTouchEnd = this.onTouchEnd.bind(this);
        this.onTouchCancel = this.onTouchCancel.bind(this);
        this.bindTouchEvent();
    }
    TouchHandler.prototype.getDirection = function (x, y) {
        if (x > y && x > MIN_DISTANCE) {
            return 'horizontal';
        }
        if (y > x && y > MIN_DISTANCE) {
            return 'vertical';
        }
        return '';
    };
    TouchHandler.prototype.touchStart = function (event) {
        var touch = event.touches[0];
        this.resetTouchState();
        this.startX = touch.clientX;
        this.startY = touch.clientY;
    };
    TouchHandler.prototype.touchMove = function (event) {
        var touch = event.touches[0];
        this.deltaX = touch.clientX - this.startX;
        this.deltaY = touch.clientY - this.startY;
        this.offsetX = Math.abs(this.deltaX);
        this.offsetY = Math.abs(this.deltaY);
        this.direction = this.direction || this.getDirection(this.offsetX, this.offsetY);
    };
    TouchHandler.prototype.onTouchStart = function (event) {
        var _a = this.opt, onBeforeTouchStart = _a.onBeforeTouchStart, onTouchStart = _a.onTouchStart;
        if (onBeforeTouchStart && onBeforeTouchStart(event) === false) {
            return;
        }
        this.touchStart(event);
        onTouchStart && onTouchStart(event);
    };
    TouchHandler.prototype.onTouchMove = function (event) {
        var _a = this.opt, onBeforeTouchMove = _a.onBeforeTouchMove, onTouchMove = _a.onTouchMove;
        if (onBeforeTouchMove && onBeforeTouchMove(event) === false) {
            return;
        }
        this.touchMove(event);
        onTouchMove && onTouchMove(event);
    };
    TouchHandler.prototype.onTouchEnd = function (event) {
        var onTouchEnd = this.opt.onTouchEnd;
        onTouchEnd && onTouchEnd(event);
    };
    TouchHandler.prototype.onTouchCancel = function (event) {
        var onTouchCancel = this.opt.onTouchCancel;
        onTouchCancel && onTouchCancel(event);
    };
    TouchHandler.prototype.resetTouchState = function () {
        this.direction = '';
        this.deltaX = 0;
        this.deltaY = 0;
        this.offsetX = 0;
        this.offsetY = 0;
    };
    TouchHandler.prototype.bindTouchEvent = function () {
        this.el.addEventListener('touchstart', this.onTouchStart, false);
        this.el.addEventListener('touchmove', this.onTouchMove, false);
        this.el.addEventListener('touchend', this.onTouchEnd, false);
        this.el.addEventListener('touchcancel', this.onTouchCancel, false);
    };
    Object.defineProperty(TouchHandler.prototype, "state", {
        get: function () {
            return {
                startX: this.startX,
                startY: this.startY,
                deltaX: this.deltaX,
                deltaY: this.deltaY,
                offsetX: this.offsetX,
                offsetY: this.offsetY,
                direction: this.direction,
            };
        },
        enumerable: false,
        configurable: true
    });
    TouchHandler.prototype.destroy = function () {
        this.el.removeEventListener('touchstart', this.onTouchStart);
        this.el.removeEventListener('touchmove', this.onTouchMove);
        this.el.removeEventListener('touchend', this.onTouchEnd);
        this.el.removeEventListener('touchcancel', this.onTouchCancel);
        this.el = null;
    };
    return TouchHandler;
}());
export { TouchHandler };
