"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ActionSheet = void 0;
var jsx_runtime_1 = require("react/jsx-runtime");
var clsx_1 = __importDefault(require("clsx"));
var popup_1 = require("../popup");
var loading_1 = require("../loading");
var icon_1 = require("../icon");
var bem_1 = require("../utils/bem");
var constant_1 = require("../utils/constant");
require("./index.scss");
var bem = bem_1.createBEM('pant-action-sheet');
var ActionSheet = function (props) {
    var show = props.show, title = props.title, cancelText = props.cancelText;
    function Header() {
        if (title) {
            return (jsx_runtime_1.jsxs("div", __assign({ className: bem('header') }, { children: [title, jsx_runtime_1.jsx(icon_1.Icon, { name: props.closeIcon, className: bem('close'), onClick: props.onCancel }, void 0)] }), void 0));
        }
    }
    function Content() {
        if (props.children) {
            return jsx_runtime_1.jsx("div", __assign({ className: bem('content') }, { children: props.children }), void 0);
        }
    }
    function Option(item, index) {
        var disabled = item.disabled, loading = item.loading;
        function onClickOption(event) {
            event.stopPropagation();
            if (disabled || loading) {
                return;
            }
            props.onSelect && props.onSelect(item, index);
        }
        function OptionContent() {
            if (loading) {
                return jsx_runtime_1.jsx(loading_1.Loading, { size: "20px" }, void 0);
            }
            return [
                jsx_runtime_1.jsx("span", __assign({ className: bem('name') }, { children: item.name }), "name"),
                item.subname && (jsx_runtime_1.jsx("span", __assign({ className: bem('subname') }, { children: item.subname }), "subname")),
            ];
        }
        return (jsx_runtime_1.jsx("button", __assign({ type: "button", className: clsx_1.default(bem('item', { disabled: disabled, loading: loading }), item.className, constant_1.BORDER_TOP), style: { color: item.color }, onClick: onClickOption }, { children: OptionContent() }), index));
    }
    function CancelText() {
        if (cancelText) {
            return (jsx_runtime_1.jsx("button", __assign({ type: "button", className: bem('cancel'), onClick: props.onCancel }, { children: cancelText }), void 0));
        }
    }
    var Description = props.description && jsx_runtime_1.jsx("div", __assign({ className: bem('description') }, { children: props.description }), void 0);
    return (jsx_runtime_1.jsx(popup_1.Popup, __assign({ className: bem(), show: show, position: props.position || 'bottom', zIndex: props.zIndex, round: props.round, overlay: props.overlay, duration: props.duration, lazyRender: props.lazyRender, closeOnClickOverlay: props.closeOnClickOverlay, safeAreaInsetBottom: props.safeAreaInsetBottom, onClickClose: props.onCancel, onOpened: show ? props.onOpened : null, onClosed: show ? null : props.onClosed }, { children: jsx_runtime_1.jsxs(jsx_runtime_1.Fragment, { children: [Header(), Description, props.actions && props.actions.map(Option), Content(), CancelText()] }, void 0) }), void 0));
};
exports.ActionSheet = ActionSheet;
exports.ActionSheet.defaultProps = {
    round: true,
    safeAreaInsetBottom: true,
    closeOnClickOverlay: true,
};
