"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Button = void 0;
var jsx_runtime_1 = require("react/jsx-runtime");
var clsx_1 = __importDefault(require("clsx"));
var constant_1 = require("../utils/constant");
var bem_1 = require("../utils/bem");
var icon_1 = require("../icon");
var loading_1 = require("../loading");
require("./index.scss");
var bem = bem_1.createBEM('pant-button');
var Button = function (props) {
    var _a;
    var url = props.url, icon = props.icon, type = props.type, color = props.color, plain = props.plain, disabled = props.disabled, loading = props.loading, hairline = props.hairline, loadingText = props.loadingText;
    var style = {};
    if (color) {
        style.color = plain ? color : constant_1.WHITE;
        if (!plain) {
            // Use background instead of backgroundColor to make linear-gradient work
            style.background = color;
        }
        // hide border when color is linear-gradient
        if (color.indexOf('gradient') !== -1) {
            style.border = '0';
        }
        else {
            style.borderColor = color;
        }
    }
    style = __assign(__assign({}, style), props.style);
    function onClick(event) {
        if (!loading && !disabled && props.onClick) {
            props.onClick(event);
        }
    }
    var classes = [
        bem([
            type,
            props.size,
            {
                plain: plain,
                loading: loading,
                disabled: disabled,
                hairline: hairline,
                block: props.block,
                round: props.round,
                square: props.square,
            },
        ]),
        (_a = {}, _a[constant_1.BORDER_SURROUND] = hairline, _a),
        props.className,
    ];
    function Content() {
        var content = [];
        if (loading) {
            content.push(jsx_runtime_1.jsx(loading_1.Loading, { className: bem('loading'), size: props.loadingSize, type: props.loadingType, color: "currentColor" }, "loading"));
        }
        else if (icon) {
            content.push(jsx_runtime_1.jsx(icon_1.Icon, { name: icon, className: bem('icon') }, "icon"));
        }
        var text;
        if (loading) {
            text = loadingText;
        }
        else {
            text = props.children || props.text;
        }
        if (text) {
            content.push(jsx_runtime_1.jsx("span", __assign({ className: bem('text') }, { children: text }), "text"));
        }
        return content;
    }
    if (url) {
        return (jsx_runtime_1.jsx("a", __assign({ href: url, style: style, className: clsx_1.default(classes), disabled: disabled, target: props.targetBlank ? '_blank' : undefined }, { children: Content() }), void 0));
    }
    else {
        return (jsx_runtime_1.jsx("button", __assign({ style: style, className: clsx_1.default(classes), type: props.nativeType, disabled: disabled, onClick: onClick }, { children: Content() }), void 0));
    }
};
exports.Button = Button;
exports.Button.defaultProps = {
    type: 'default',
    size: 'normal',
    loadingSize: '20px',
};
