"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Cascader = void 0;
var jsx_runtime_1 = require("react/jsx-runtime");
var react_1 = __importDefault(require("react"));
var clsx_1 = __importDefault(require("clsx"));
var lodash_es_1 = require("lodash-es");
var icon_1 = require("../icon");
var loading_1 = require("../loading");
var toast_1 = require("../toast");
var toolbar_1 = require("../popup/toolbar");
var locale_1 = require("../locale");
var event_bus_1 = require("../utils/event-bus");
var utils_1 = require("../utils");
var bem_1 = require("../utils/bem");
var cascader_column_1 = require("./cascader-column");
require("./index.scss");
var bem = bem_1.createBEM('pant-cascader');
var MIN_COLUMN_WIDTH = 100;
var MAX_COLUMN_WIDTH = 300;
var LAST_COLUMN_EXTRA_WIDTH = 30;
var Cascader = /** @class */ (function (_super) {
    __extends(Cascader, _super);
    function Cascader(props) {
        var _this = _super.call(this, props) || this;
        _this.contentRef = react_1.default.createRef();
        var defaultValue = (Array.isArray(props.defaultValue) && Array.isArray(props.defaultValue[0])
            ? props.defaultValue
            : Array.isArray(props.defaultValue) && typeof props.defaultValue[0] === 'string'
                ? [props.defaultValue]
                : undefined) || [];
        var currentValue = defaultValue.length ? defaultValue[0].slice(0, -1) : [];
        _this.state = {
            pickerValue: __spreadArray([], defaultValue),
            rollbackPickerValue: __spreadArray([], defaultValue),
            currentValue: __spreadArray([], currentValue),
            rollbackCurrentValue: __spreadArray([], currentValue),
            contentWidth: 0,
            backSteps: 0,
            loading: false,
            data: props.data || [],
            prevProps: props,
        };
        _this.onPopupOpened = _this.onPopupOpened.bind(_this);
        _this.isItemSelected = _this.isItemSelected.bind(_this);
        _this.hasChildrenSelected = _this.hasChildrenSelected.bind(_this);
        _this.onClickItem = _this.onClickItem.bind(_this);
        _this.showLoading = _this.showLoading.bind(_this);
        _this.hideLoading = _this.hideLoading.bind(_this);
        _this.backOneStep = _this.backOneStep.bind(_this);
        _this.forwardOneStep = _this.forwardOneStep.bind(_this);
        _this.normalizeItem = _this.normalizeItem.bind(_this);
        _this.onCancel = _this.onCancel.bind(_this);
        _this.onConfirm = _this.onConfirm.bind(_this);
        return _this;
    }
    Cascader.getDerivedStateFromProps = function (props, state) {
        var prevProps = state.prevProps;
        if (props.data !== prevProps.data || props.defaultValue !== prevProps.defaultValue) {
            var defaultValue = (Array.isArray(props.defaultValue) && Array.isArray(props.defaultValue[0])
                ? props.defaultValue
                : Array.isArray(props.defaultValue) && typeof props.defaultValue[0] === 'string'
                    ? [props.defaultValue]
                    : undefined) || [];
            var currentValue = defaultValue.length ? defaultValue[0].slice(0, -1) : [];
            return __assign(__assign({}, state), { pickerValue: __spreadArray([], defaultValue), rollbackPickerValue: __spreadArray([], defaultValue), currentValue: __spreadArray([], currentValue), rollbackCurrentValue: __spreadArray([], currentValue), data: props.data || [], prevProps: props });
        }
        return null;
    };
    Cascader.prototype.componentDidMount = function () {
        var _this = this;
        var _a = this.props, _popupId = _a._popupId, data = _a.data, onLoad = _a.onLoad;
        event_bus_1.eventBus.on('popup.opened', this.onPopupOpened);
        if (!_popupId) {
            this.setState({
                contentWidth: this.contentRef.current.clientWidth,
            });
        }
        if (onLoad && !data.length) {
            this.showLoading();
            onLoad([])
                .then(function (data) {
                _this.setState({ data: data || [] });
            })
                .finally(this.hideLoading);
        }
    };
    Cascader.prototype.componentWillUnmount = function () {
        event_bus_1.eventBus.off('popup.opened', this.onPopupOpened);
    };
    Cascader.prototype.onPopupOpened = function (id) {
        if (id === this.props._popupId) {
            this.setState({ contentWidth: this.contentRef.current.clientWidth });
        }
    };
    Cascader.prototype.getValue = function () {
        var maxSelection = this.props.maxSelection;
        var pickerValue = this.state.pickerValue;
        if (maxSelection === 1) {
            return pickerValue[0];
        }
        return __spreadArray([], pickerValue);
    };
    Cascader.prototype.clearValue = function (cb) {
        this.setState({ pickerValue: [], rollbackPickerValue: [] }, cb);
    };
    Cascader.prototype.showLoading = function () {
        this.setState({ loading: true });
    };
    Cascader.prototype.hideLoading = function () {
        this.setState({ loading: false });
    };
    Cascader.prototype.genToolbar = function (top) {
        var props = this.props;
        if (!props.showToolbar) {
            if (!props.title || !top) {
                return;
            }
            if (top) {
                return (jsx_runtime_1.jsx("div", __assign({ className: bem('toolbar', { title: true }) }, { children: jsx_runtime_1.jsx("div", __assign({ className: clsx_1.default(bem('title'), 'pant-ellipsis') }, { children: props.title }), "title") }), void 0));
            }
        }
        if (props.toolbarPosition === 'top') {
            if (top) {
                return (jsx_runtime_1.jsx(toolbar_1.PopupToolbar, { title: props.title, cancelButtonText: props.cancelButtonText || locale_1.i18n().cancel, confirmButtonText: props.confirmButtonText || locale_1.i18n().confirm, onCancel: this.onCancel, onConfirm: this.onConfirm }, void 0));
            }
        }
        else {
            if (top) {
                if (props.title) {
                    return jsx_runtime_1.jsx(toolbar_1.PopupToolbar, { title: props.title }, void 0);
                }
            }
            else {
                return (jsx_runtime_1.jsx(toolbar_1.PopupToolbar, { cancelButtonText: props.cancelButtonText || locale_1.i18n().cancel, confirmButtonText: props.confirmButtonText || locale_1.i18n().confirm, onCancel: this.onCancel, onConfirm: this.onConfirm }, void 0));
            }
        }
    };
    Cascader.prototype.onConfirm = function () {
        var _this = this;
        var _a = this.state, pickerValue = _a.pickerValue, currentValue = _a.currentValue;
        this.setState({ rollbackPickerValue: __spreadArray([], pickerValue), rollbackCurrentValue: __spreadArray([], currentValue) }, function () {
            var _a = _this.props, closePopup = _a.closePopup, onConfirm = _a.onConfirm;
            onConfirm && onConfirm(_this.getValue());
            closePopup && closePopup(true);
        });
    };
    Cascader.prototype.onCancel = function () {
        var _this = this;
        var _a = this.state, rollbackPickerValue = _a.rollbackPickerValue, rollbackCurrentValue = _a.rollbackCurrentValue;
        this.setState({ pickerValue: __spreadArray([], rollbackPickerValue), currentValue: __spreadArray([], rollbackCurrentValue) }, function () {
            var _a = _this.props, closePopup = _a.closePopup, onCancel = _a.onCancel;
            onCancel && onCancel();
            closePopup && closePopup();
        });
    };
    Cascader.prototype.normalizeItem = function (item) {
        var _a = this.props, valueKey = _a.valueKey, labelKey = _a.labelKey;
        var value = lodash_es_1.get(item, valueKey, '') + '';
        return __assign(__assign({}, item), { value: value, label: lodash_es_1.get(item, labelKey, value) });
    };
    Cascader.prototype.isSameValue = function (a, b) {
        if (!a || !b) {
            return false;
        }
        return a.length === b.length && a.every(function (item) { return b.includes(item); });
    };
    Cascader.prototype.isContainValue = function (a, b) {
        if (!a || !b) {
            return false;
        }
        return a.every(function (item) { return b.includes(item); });
    };
    Cascader.prototype.isItemSelected = function (columnIndex, item) {
        var _this = this;
        var _a = this.state, currentValue = _a.currentValue, pickerValue = _a.pickerValue;
        var targetValue = __spreadArray(__spreadArray([], currentValue.slice(0, columnIndex)), [item.value]);
        return pickerValue.some(function (value) { return _this.isSameValue(targetValue, value); });
    };
    Cascader.prototype.hasChildrenSelected = function (columnIndex, item) {
        var _this = this;
        var _a = this.state, currentValue = _a.currentValue, pickerValue = _a.pickerValue;
        var targetValue = __spreadArray(__spreadArray([], currentValue.slice(0, columnIndex)), [item.value]);
        return pickerValue.some(function (value) { return _this.isContainValue(targetValue, value); });
    };
    Cascader.prototype.onClickItem = function (columnIndex, item) {
        var _this = this;
        var _a = this.props, maxSelection = _a.maxSelection, maxSelectionMsg = _a.maxSelectionMsg, changeOnSelect = _a.changeOnSelect, onChange = _a.onChange, onLoad = _a.onLoad;
        var _b = this.state, currentValue = _b.currentValue, pickerValue = _b.pickerValue, data = _b.data;
        var newValue = __spreadArray(__spreadArray([], currentValue.slice(0, columnIndex)), [item.value]);
        if (item.children) {
            if (!item.children.length && onLoad) {
                this.showLoading();
                onLoad(__spreadArray([], newValue))
                    .then(function (items) {
                    var res = data;
                    var item;
                    var value = __spreadArray([], newValue);
                    var _loop_1 = function () {
                        var v = value.shift();
                        item = res.find(function (item) { return _this.normalizeItem(item).value === v; });
                        res = item && item.children;
                    };
                    while (res && value.length) {
                        _loop_1();
                    }
                    item.children = items;
                    if (changeOnSelect && maxSelection === 1) {
                        _this.setState({ pickerValue: [newValue], data: data, backSteps: 0, currentValue: newValue }, function () {
                            onChange && onChange(_this.getValue());
                        });
                    }
                    else {
                        _this.setState({ data: data, backSteps: 0, currentValue: newValue });
                    }
                })
                    .finally(this.hideLoading);
            }
            else {
                if (changeOnSelect && maxSelection === 1) {
                    this.setState({ pickerValue: [newValue], backSteps: 0, currentValue: newValue }, function () {
                        onChange && onChange(_this.getValue());
                    });
                }
                else {
                    this.setState({ backSteps: 0, currentValue: newValue });
                }
            }
        }
        else {
            var newPickerValue = void 0;
            if (maxSelection === 1) {
                if (this.isSameValue(pickerValue[0], newValue)) {
                    newPickerValue = [];
                }
                else {
                    newPickerValue = [newValue];
                }
            }
            else {
                newPickerValue = pickerValue.filter(function (value) { return !_this.isSameValue(value, newValue); });
                if (newPickerValue.length === pickerValue.length) {
                    if (pickerValue.length >= maxSelection) {
                        toast_1.toast(utils_1.interpolate(maxSelectionMsg || locale_1.i18n().maxSelection, [maxSelection]));
                        return;
                    }
                    else {
                        newPickerValue = __spreadArray(__spreadArray([], pickerValue), [newValue]);
                    }
                }
            }
            this.setState({ pickerValue: newPickerValue }, function () {
                onChange && onChange(_this.getValue());
            });
        }
    };
    Cascader.prototype.getColumnWidth = function () {
        return Math.max(MIN_COLUMN_WIDTH, Math.min(this.props.columnWidth, MAX_COLUMN_WIDTH));
    };
    Cascader.prototype.getOffset = function (columns) {
        var _a = this.state, contentWidth = _a.contentWidth, backSteps = _a.backSteps;
        if (!contentWidth) {
            return 0;
        }
        var columnWidth = this.getColumnWidth();
        return Math.min(0, contentWidth - columnWidth * (columns - backSteps) - (backSteps > 0 ? 0 : LAST_COLUMN_EXTRA_WIDTH));
    };
    Cascader.prototype.getColumnItems = function (values) {
        var _this = this;
        var items = this.state.data;
        var _loop_2 = function () {
            var value = values.shift();
            var item = items.find(function (item) { return _this.normalizeItem(item).value === value; });
            items = item && item.children;
        };
        while (items && values.length) {
            _loop_2();
        }
        return items;
    };
    Cascader.prototype.genColumns = function () {
        var _a = this.props, maxSelection = _a.maxSelection, checkedIcon = _a.checkedIcon, uncheckedIcon = _a.uncheckedIcon;
        var _b = this.state, data = _b.data, currentValue = _b.currentValue, contentWidth = _b.contentWidth;
        var len = currentValue.length;
        var columnWidth = this.getColumnWidth();
        var res = [];
        res.push(jsx_runtime_1.jsx(cascader_column_1.CascaderColumn, { maxSelection: maxSelection, index: 0, value: currentValue[0], width: len ? columnWidth : contentWidth, checkedIcon: checkedIcon, uncheckedIcon: uncheckedIcon, isItemSelected: this.isItemSelected, hasChildrenSelected: this.hasChildrenSelected, normalizeItem: this.normalizeItem, onClick: this.onClickItem, items: data }, "root"));
        for (var i = 0; i < len; i++) {
            var items = this.getColumnItems(currentValue.slice(0, i + 1));
            if (!items || !items.length) {
                break;
            }
            var width = void 0;
            if (i === len - 1) {
                var extra = contentWidth - (len + 1) * columnWidth - LAST_COLUMN_EXTRA_WIDTH;
                width = columnWidth + LAST_COLUMN_EXTRA_WIDTH + Math.max(extra, 0);
            }
            else {
                width = columnWidth;
            }
            res.push(jsx_runtime_1.jsx(cascader_column_1.CascaderColumn, { maxSelection: maxSelection, index: i + 1, value: currentValue[i + 1], width: width, checkedIcon: checkedIcon, uncheckedIcon: uncheckedIcon, isItemSelected: this.isItemSelected, hasChildrenSelected: this.hasChildrenSelected, normalizeItem: this.normalizeItem, onClick: this.onClickItem, items: items }, currentValue[i]));
        }
        return res;
    };
    Cascader.prototype.backOneStep = function () {
        this.setState({ backSteps: this.state.backSteps + 1 });
    };
    Cascader.prototype.forwardOneStep = function () {
        var backSteps = this.state.backSteps;
        if (backSteps > 0) {
            this.setState({ backSteps: backSteps - 1 });
        }
    };
    Cascader.prototype.genBackBtn = function () {
        return (jsx_runtime_1.jsx("div", __assign({ className: bem('back'), onClick: this.backOneStep }, { children: jsx_runtime_1.jsx(icon_1.Icon, { name: "arrow-left" }, void 0) }), void 0));
    };
    Cascader.prototype.genForwardBtn = function () {
        return (jsx_runtime_1.jsx("div", __assign({ className: bem('forward'), onClick: this.forwardOneStep }, { children: jsx_runtime_1.jsx(icon_1.Icon, { name: "arrow" }, void 0) }), void 0));
    };
    Cascader.prototype.render = function () {
        var _a = this.props, _popupId = _a._popupId, height = _a.height;
        var _b = this.state, loading = _b.loading, backSteps = _b.backSteps;
        var columns = this.genColumns();
        var offset = this.getOffset(columns.length);
        return (jsx_runtime_1.jsxs("div", __assign({ className: bem(), style: { height: _popupId ? 'auto' : height + 'px' } }, { children: [this.genToolbar(true), jsx_runtime_1.jsxs("div", __assign({ ref: this.contentRef, className: bem('content') }, { children: [jsx_runtime_1.jsx("div", __assign({ className: bem('columns'), style: { transform: "translateX(" + offset + "px)" } }, { children: columns }), void 0),
                        loading ? jsx_runtime_1.jsx(loading_1.Loading, { size: "30" }, void 0) : null, offset !== 0 && this.genBackBtn(), backSteps !== 0 && this.genForwardBtn()] }), void 0), this.genToolbar()] }), void 0));
    };
    Cascader.defaultProps = {
        height: 300,
        columnWidth: 140,
        maxSelection: 1,
        showToolbar: true,
        toolbarPosition: 'top',
        valueKey: 'value',
        labelKey: 'label',
        data: [],
    };
    return Cascader;
}(react_1.default.PureComponent));
exports.Cascader = Cascader;
