"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SelectableManager = void 0;
var toast_1 = require("../../../toast");
var locale_1 = require("../../../locale");
var utils_1 = require("../../../utils");
var key_selector_1 = require("../../key-selector");
var SelectableManager = /** @class */ (function () {
    function SelectableManager(records, value, recordKey, maxSelection, maxSelectionMsg, onChange) {
        if (records === void 0) { records = []; }
        if (value === void 0) { value = []; }
        if (maxSelection === void 0) { maxSelection = 9999; }
        this.records = records;
        this.value = value;
        this.recordKey = recordKey;
        this.maxSelection = maxSelection;
        this.maxSelectionMsg = maxSelectionMsg;
        this.onChange = onChange;
    }
    SelectableManager.prototype.getValue = function () {
        return __spreadArray([], this.value);
    };
    SelectableManager.prototype.hasKey = function (key) {
        return this.value.includes(key);
    };
    SelectableManager.prototype.toggle = function (key) {
        var onChange = this.onChange;
        if (this.hasKey(key)) {
            onChange(this.value.filter(function (v) { return v !== key; }));
        }
        else {
            var _a = this, maxSelection = _a.maxSelection, maxSelectionMsg = _a.maxSelectionMsg;
            if (maxSelection === 1) {
                onChange([key]);
            }
            else if (maxSelection > this.value.length) {
                onChange(__spreadArray(__spreadArray([], this.value), [key]));
            }
            else {
                toast_1.toast(utils_1.interpolate(maxSelectionMsg || locale_1.i18n().maxSelection, [maxSelection]));
            }
        }
    };
    SelectableManager.prototype.toggleAll = function () {
        var _this = this;
        var all = this.records.map(key_selector_1.select(this.recordKey));
        var onChange = this.onChange;
        if (all.every(function (v) { return _this.value.includes(v); })) {
            onChange([]);
        }
        else {
            onChange(all.slice(0, this.maxSelection));
        }
    };
    return SelectableManager;
}());
exports.SelectableManager = SelectableManager;
