import React from 'react';
import { CellProps } from '../cell';
import './index.scss';
export declare type ValidatorFn<T> = (value: T) => Promise<string | void>;
export declare type ValidateTrigger = 'blur' | 'change';
export declare type PatternValidator = {
    readonly trigger?: ValidateTrigger[];
    readonly pattern: string | RegExp;
    readonly message: string;
};
export declare type FnValidator<T> = {
    readonly trigger?: ValidateTrigger[];
    readonly validator: ValidatorFn<T>;
};
export declare type ValidateRule<T> = PatternValidator | FnValidator<T> | ValidatorFn<T>;
export declare type FieldProps<T> = Omit<CellProps, 'onClick'> & {
    type?: 'checkbox' | 'color' | 'date' | 'datetime-local' | 'digit' | 'email' | 'file' | 'month' | 'number' | 'password' | 'range' | 'search' | 'switch' | 'tel' | 'text' | 'textarea' | 'time' | 'url' | 'week';
    name: string;
    defaultValue?: T;
    rules?: ValidateRule<T>[] | ValidatorFn<T>;
    disabled?: boolean;
    readOnly?: boolean;
    autosize?: boolean | {
        minHeight: number;
        maxHeight: number;
    };
    button?: React.ReactNode;
    icon?: React.ReactNode;
    rightIcon?: React.ReactNode;
    clearable?: boolean;
    clearTrigger?: 'always' | 'focus';
    maxlength?: number | string;
    labelWidth?: number | string;
    labelClass?: null;
    labelAlign?: 'center' | 'right';
    inputAlign?: 'center' | 'right';
    direction?: 'row' | 'column';
    bodyStyle?: Record<string, string>;
    placeholder?: string;
    errorMessage?: string;
    errorMessageAlign?: 'center' | 'right';
    showWordLimit?: boolean;
    validateTrigger?: ValidateTrigger[];
    valueFormatter?(value: any): T;
    displayValueFormatter?(value: T): React.ReactNode;
    onClosePopup?(field: Field<T>, confirm?: boolean): void;
    onInputKeyDown?(evt: React.KeyboardEvent): void;
    onInputChange?(evt: React.ChangeEvent): string | void;
    onChange?(value: T): void;
    onFocus?(evt: React.FocusEvent): void;
    onBlur?(evt: React.FocusEvent): void;
    onClick?: (evt: React.MouseEvent) => void;
};
declare type FieldState<T> = {
    isInputType: boolean;
    focused: boolean;
    showPopup: boolean;
    popupValue: T;
    value: T;
    validateMessage: string;
    prevProps: FieldProps<T>;
};
export declare class Field<T = never> extends React.PureComponent<FieldProps<T>, FieldState<T>> {
    static defaultProps: {
        clearTrigger: string;
    };
    private isPopup;
    private readonly inputRef;
    private readonly bodyRef;
    constructor(props: React.PropsWithChildren<FieldProps<T>>);
    static getDerivedStateFromProps<T>(props: React.PropsWithChildren<FieldProps<T>>, state: FieldState<T>): FieldState<T>;
    componentDidMount(): void;
    componentDidUpdate(): void;
    private isEmptyValue;
    private get showClear();
    private get showError();
    getName(): string;
    getRawValue(): any;
    getValue(): T;
    private isCustomChild;
    private onFocus;
    private onBlur;
    private onCustomChange;
    private onInputChange;
    private onCheckboxClick;
    private onSwitchClick;
    openPopup(): void;
    closePopup(confirm?: boolean): void;
    private onPopupControlClick;
    private clearInput;
    private adjustSize;
    private validateWithTrigger;
    private validate;
    private formatReturnValue;
    private formatDisplayValue;
    doValidate(): Promise<string | void>;
    private genInput;
    private genRightIcon;
    private genWordLimit;
    private genMessage;
    render(): JSX.Element;
}
export {};
