"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NumberKeyboard = void 0;
var jsx_runtime_1 = require("react/jsx-runtime");
var react_1 = __importDefault(require("react"));
var clsx_1 = __importDefault(require("clsx"));
var dom_1 = require("../utils/dom");
var bem_1 = require("../utils/bem");
var transition_1 = require("../transition");
var key_1 = require("./key");
require("./index.scss");
var bem = bem_1.createBEM('pant-number-keyboard');
var NumberKeyboard = /** @class */ (function (_super) {
    __extends(NumberKeyboard, _super);
    function NumberKeyboard(props) {
        var _this = _super.call(this, props) || this;
        _this.state = {
            prevShow: false,
            prevTitle: '',
            transition: true,
        };
        _this.onClickOutside = _this.onClickOutside.bind(_this);
        _this.onBlur = _this.onBlur.bind(_this);
        _this.onAnimationEnd = _this.onAnimationEnd.bind(_this);
        _this.onClose = _this.onClose.bind(_this);
        _this.onPress = _this.onPress.bind(_this);
        return _this;
    }
    NumberKeyboard.getDerivedStateFromProps = function (nextProps, state) {
        var title = nextProps.title, show = nextProps.show;
        var prevTitle = state.prevTitle, prevShow = state.prevShow;
        if (title !== prevTitle || show !== prevShow) {
            return {
                prevTitle: title,
                prevShow: show,
                transition: true,
            };
        }
        else {
            return null;
        }
    };
    NumberKeyboard.prototype.componentDidMount = function () {
        if (this.props.hideOnClickOutside) {
            document.body.addEventListener('touchstart', this.onClickOutside, false);
        }
    };
    NumberKeyboard.prototype.componentWillUnmount = function () {
        if (this.props.hideOnClickOutside) {
            document.body.removeEventListener('touchstart', this.onClickOutside);
        }
    };
    NumberKeyboard.prototype.onClickOutside = function (event) {
        if (dom_1.closest(event.target, '.pant-number-keyboard', true)) {
            return;
        }
        this.props.show && this.props.onBlur();
    };
    NumberKeyboard.prototype.onTouchStart = function (event) {
        event.stopPropagation();
    };
    NumberKeyboard.prototype.onBlur = function () {
        this.props.show && this.props.onBlur();
    };
    NumberKeyboard.prototype.onClose = function () {
        this.props.onClose && this.props.onClose();
        this.onBlur();
    };
    NumberKeyboard.prototype.onPress = function (text, type) {
        if (text === '') {
            if (type === 'extra') {
                this.onBlur();
            }
            return;
        }
        if (type === 'delete') {
            this.props.onDelete && this.props.onDelete();
        }
        else if (type === 'close') {
            this.onClose();
        }
        else {
            this.props.onInput && this.props.onInput(text);
        }
    };
    NumberKeyboard.prototype.genBasicKeys = function () {
        var keys = [];
        for (var i = 1; i <= 9; i++) {
            keys.push({ text: i });
        }
        return keys;
    };
    NumberKeyboard.prototype.genDefaultKeys = function () {
        return __spreadArray(__spreadArray([], this.genBasicKeys()), [
            { text: this.props.extraKey, type: 'extra' },
            { text: 0 },
            {
                text: this.props.showDeleteKey ? this.props.deleteButtonText : '',
                type: this.props.showDeleteKey ? 'delete' : '',
            },
        ]);
    };
    NumberKeyboard.prototype.genCustomKeys = function () {
        var keys = this.genBasicKeys();
        var extraKey = this.props.extraKey;
        var extraKeys = Array.isArray(extraKey) ? extraKey : [extraKey];
        if (extraKeys.length === 1) {
            keys.push({ text: 0, wider: true }, { text: extraKey[0], type: 'extra' });
        }
        else if (extraKeys.length === 2) {
            keys.push({ text: extraKey[0], type: 'extra' }, { text: 0 }, { text: extraKey[1], type: 'extra' });
        }
        return keys;
    };
    NumberKeyboard.prototype.onAnimationEnd = function () {
        if (this.props.show) {
            this.props.onShow && this.props.onShow();
        }
        else {
            this.props.onHide && this.props.onHide();
        }
    };
    NumberKeyboard.prototype.genTitle = function () {
        var _a = this.props, title = _a.title, theme = _a.theme, closeButtonText = _a.closeButtonText;
        var customTitle;
        if (title && typeof title !== 'string') {
            customTitle = title;
        }
        var showClose = closeButtonText && theme === 'default';
        var showTitle = title || showClose || customTitle;
        if (!showTitle) {
            return;
        }
        return (jsx_runtime_1.jsxs("div", __assign({ className: bem('header') }, { children: [customTitle ? customTitle : title ? jsx_runtime_1.jsx("h2", __assign({ className: bem('title') }, { children: title }), void 0) : null,
                showClose && (jsx_runtime_1.jsx("button", __assign({ type: "button", className: bem('close'), onClick: this.onClose }, { children: closeButtonText }), void 0))] }), void 0));
    };
    NumberKeyboard.prototype.genKeys = function () {
        var _this = this;
        var keys = [];
        if (this.props.theme === 'custom') {
            keys = this.genCustomKeys();
        }
        else {
            keys = this.genDefaultKeys();
        }
        return keys.map(function (key, i) { return (jsx_runtime_1.jsx(key_1.Key, { text: key.text, type: key.type, wider: key.wider, onPress: _this.onPress.bind(_this) }, i)); });
    };
    NumberKeyboard.prototype.genSidebar = function () {
        var _a = this.props, theme = _a.theme, showDeleteKey = _a.showDeleteKey, deleteButtonText = _a.deleteButtonText, closeButtonText = _a.closeButtonText;
        if (theme === 'custom') {
            return (jsx_runtime_1.jsxs("div", __assign({ className: bem('sidebar') }, { children: [showDeleteKey && jsx_runtime_1.jsx(key_1.Key, { large: true, text: deleteButtonText, type: "delete", onPress: this.onPress.bind(this) }, void 0),
                    jsx_runtime_1.jsx(key_1.Key, { large: true, text: closeButtonText, type: "close", color: "blue", onPress: this.onPress.bind(this) }, void 0)] }), void 0));
        }
    };
    NumberKeyboard.prototype.render = function () {
        var _a = this.props, safeAreaInsetBottom = _a.safeAreaInsetBottom, zIndex = _a.zIndex;
        var _b = this.state, prevShow = _b.prevShow, transition = _b.transition;
        var Title = this.genTitle();
        return (jsx_runtime_1.jsx(transition_1.Transition, __assign({ customName: transition ? 'slide-up' : '', stage: prevShow ? 'enter' : 'leave' }, { children: jsx_runtime_1.jsxs("div", __assign({ style: { zIndex: zIndex }, className: clsx_1.default(bem({
                    unfit: !safeAreaInsetBottom,
                    'with-title': Title,
                    show: prevShow,
                })), onTouchStart: this.onTouchStart, onAnimationEnd: this.onAnimationEnd }, { children: [Title, jsx_runtime_1.jsxs("div", __assign({ className: bem('body') }, { children: [jsx_runtime_1.jsx("div", __assign({ className: bem('keys') }, { children: this.genKeys() }), void 0), this.genSidebar()] }), void 0)] }), void 0) }), void 0));
    };
    NumberKeyboard.defaultProps = {
        title: '键盘标题',
        theme: 'default',
        show: false,
        zIndex: 100,
        extraKey: '',
        closeButtonText: '完成',
        showDeleteKey: true,
        hideOnClickOutside: true,
        safeAreaInsetBottom: true,
    };
    return NumberKeyboard;
}(react_1.default.PureComponent));
exports.NumberKeyboard = NumberKeyboard;
