import React from 'react';
import { PickerColumn } from './picker-column';
import './index.scss';
export declare type ToolbarPosition = 'top' | 'bottom';
export declare type StandardColumnItem = {
    value: string;
    label: string;
};
export declare type ColumnItem = {
    [key: string]: string | string[] | ColumnItem[];
    value?: string;
    label?: string;
    children?: ColumnItem[];
};
export declare type PickerProps = {
    /** 可选项数据源 */
    columns?: string[] | string[][] | ColumnItem[] | ColumnItem[][];
    /** 默认的选中项 */
    defaultValue?: string | string[];
    /** 展示标题栏 */
    showToolbar?: boolean;
    /** 标题栏位置 */
    toolbarPosition?: ToolbarPosition;
    /** 标题文本 */
    title?: string;
    /** 取消按钮文本 */
    cancelButtonText?: string;
    /** 确定按钮文本 */
    confirmButtonText?: string;
    /** 行高 */
    itemHeight?: number;
    /** 窗口内可视行数 */
    visibleItemCount?: number;
    /** 自定义value的key值 */
    valueKey?: string | string[];
    /** 自定义label的key值 */
    labelKey?: string | string[];
    /** 滑动惯性滚动速度 */
    swipeDuration?: number;
    /** 加载中展示 */
    loading?: boolean;
    /** 自定义列样式 */
    columnClassName?: string;
    /** 不可选值 */
    disabledValue?: string[];
    /** 列数 */
    cols?: number;
    /** 是否联动 */
    cascade?: boolean;
    /** 选择取消后的回调 */
    onCancel?: (value: string[] | string) => void;
    /** 选择确定后的回调 */
    onConfirm?: (value: string[] | string) => void;
    closePopup?: (confirm?: boolean) => void;
    /** 点击后回调 */
    onChange?: (value: string[] | string) => void;
};
declare type PickerState = {
    formattedColumns: StandardColumnItem[][];
    children?: PickerColumn[];
    prevColumns?: string[] | string[][] | ColumnItem[] | ColumnItem[][];
    pickerValue?: string[];
};
export declare class Picker extends React.PureComponent<PickerProps, PickerState> {
    static defaultProps: {
        showToolbar: boolean;
        toolbarPosition: string;
        itemHeight: number;
        visibleItemCount: number;
        valueKey: string;
        labelKey: string;
        swipeDuration: number;
        loading: boolean;
        cols: number;
        cascade: boolean;
    };
    private priChildren;
    static getDerivedStateFromProps(nextProps: PickerProps, state: PickerState): PickerState;
    constructor(props: PickerProps);
    injectChildren(children: PickerColumn): void;
    itemPxHeight(): number;
    genToolbar(): JSX.Element;
    onCascadeChange(selectedIndex: number, columnIndex: number): void;
    onChange(selectedIndex: number, columnIndex: number): void;
    getValue(): string[] | string;
    clearValue(cb: () => void): void;
    onConfirm(): void;
    onCancel(): void;
    emit(event: 'onConfirm' | 'onCancel'): void;
    genColumns(): JSX.Element;
    genColumnItems(): JSX.Element[];
    render(): JSX.Element;
}
export {};
