"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PickerColumn = void 0;
var jsx_runtime_1 = require("react/jsx-runtime");
var react_1 = __importDefault(require("react"));
var clsx_1 = __importDefault(require("clsx"));
var bem_1 = require("../utils/bem");
var event_1 = require("../utils/event");
var touch_handler_1 = require("../utils/touch-handler");
var number_1 = require("../utils/number");
var constant_1 = require("./constant");
require("./index.scss");
function getElementTranslateY(element) {
    var style = window.getComputedStyle(element);
    var transform = style.transform || style.webkitTransform;
    var translateY = transform.slice(7, transform.length - 1).split(', ')[5];
    return Number(translateY);
}
var getIndex = function (options, value) {
    var index = 0;
    options.forEach(function (item, itemIndex) {
        if (item.value === value) {
            index = itemIndex;
        }
    });
    return index;
};
var bem = bem_1.createBEM('pant-picker-column');
var PickerColumn = /** @class */ (function (_super) {
    __extends(PickerColumn, _super);
    function PickerColumn(props) {
        var _this = _super.call(this, props) || this;
        _this.listRef = react_1.default.createRef();
        _this.state = {
            currentIndex: 0,
            offset: 0,
            duration: 0,
            baseOffset: (_this.props.itemHeight * (_this.props.visibleItemCount - 1)) / 2,
            moving: false,
            prevOptions: [],
            prevValue: '',
        };
        _this.setIndex = _this.setIndex.bind(_this);
        return _this;
    }
    PickerColumn.prototype.componentDidMount = function () {
        var listEl = this.listRef.current;
        this.props.injectChildren(this);
        this.touchHandler = new touch_handler_1.TouchHandler(listEl, {
            onTouchStart: this.onTouchStart.bind(this),
            onTouchMove: this.onTouchMove.bind(this),
            onTouchEnd: this.onTouchEnd.bind(this),
        });
        listEl.addEventListener('transitionend', this.onTransitionEnd.bind(this), false);
    };
    PickerColumn.prototype.componentWillUnmount = function () {
        this.touchHandler.destroy();
        this.touchHandler = null;
    };
    PickerColumn.getDerivedStateFromProps = function (nextProps, state) {
        var options = nextProps.options, value = nextProps.value, itemHeight = nextProps.itemHeight;
        var prevOptions = state.prevOptions, prevValue = state.prevValue;
        if (options !== prevOptions || value !== prevValue) {
            return __assign(__assign({}, state), { currentIndex: getIndex(options, value), offset: -getIndex(options, value) * itemHeight, prevOptions: options, prevValue: value });
        }
        return null;
    };
    PickerColumn.prototype.onClickItem = function (index) {
        if (this.state.moving) {
            return;
        }
        this.transitionEndTrigger = null;
        this.setState({
            duration: constant_1.DEFAULT_DURATION,
        });
        this.setIndex(index);
    };
    // 校正index，使其跳过不可选
    PickerColumn.prototype.adjustIndex = function (index) {
        var _a = this.props, options = _a.options, disabledValue = _a.disabledValue;
        var length = options.length;
        index = number_1.range(index, 0, length);
        for (var i = index; i < length; i++) {
            if (options[i].value !== disabledValue)
                return i;
        }
        for (var i = index - 1; i >= 0; i--) {
            if (options[i].value !== disabledValue)
                return i;
        }
    };
    // 点击后修改offset，调整列表位置
    PickerColumn.prototype.setIndex = function (index) {
        var props = this.props;
        // 普通点击时，touchEnd触发在click事件之前，那时index并未改变，无需进行下面逻辑
        if (index === this.state.currentIndex && this.state.offset === -index * props.itemHeight) {
            return;
        }
        index = this.adjustIndex(index);
        if (index === undefined) {
            return;
        }
        var offset = -index * props.itemHeight;
        var trigger = function () {
            props.onChange(index);
        };
        // trigger the change event after transitionend when moving
        if (this.state.moving && offset !== this.state.offset) {
            this.transitionEndTrigger = trigger;
        }
        else {
            trigger();
        }
        this.setState({
            offset: offset,
            currentIndex: index,
        });
    };
    PickerColumn.prototype.getIndexByOffset = function (offset) {
        return number_1.range(Math.round(-offset / this.props.itemHeight), 0, this.props.options.length - 1);
    };
    PickerColumn.prototype.onTouchStart = function () {
        var _this = this;
        if (this.state.moving) {
            var translateY = getElementTranslateY(this.listRef.current);
            this.setState({
                offset: Math.min(0, translateY - this.state.baseOffset),
            }, function () {
                _this.startOffset = _this.state.offset;
            });
        }
        else {
            this.startOffset = this.state.offset;
        }
        this.setState({
            duration: 0,
        });
        this.transitionEndTrigger = null;
        this.touchStartTime = Date.now();
        this.momentumOffset = this.startOffset;
    };
    PickerColumn.prototype.onTouchMove = function (event) {
        var _a = this.props, itemHeight = _a.itemHeight, options = _a.options;
        var _b = this.touchHandler.state, direction = _b.direction, deltaY = _b.deltaY;
        if (direction === 'vertical') {
            this.setState({
                moving: true,
            });
            event_1.preventDefaultAndStopPropagation(event);
        }
        this.setState({
            offset: number_1.range(this.startOffset + deltaY, -(options.length * itemHeight), itemHeight),
        });
        var now = Date.now();
        if (now - this.touchStartTime > constant_1.MOMENTUM_LIMIT_TIME) {
            this.touchStartTime = now;
            this.momentumOffset = this.state.offset;
        }
    };
    PickerColumn.prototype.onTouchEnd = function () {
        var _this = this;
        var distance = this.state.offset - this.momentumOffset;
        var duration = Date.now() - this.touchStartTime;
        var allowMomentum = duration < constant_1.MOMENTUM_LIMIT_TIME && Math.abs(distance) > constant_1.MOMENTUM_LIMIT_DISTANCE;
        if (allowMomentum) {
            this.momentum(distance, duration);
            return;
        }
        this.setState({
            duration: constant_1.DEFAULT_DURATION,
        });
        // 划出边界处理
        var index = this.getIndexByOffset(this.state.offset);
        this.setIndex(index);
        // compatible with desktop scenario
        // use setTimeout to skip the click event triggered after touchstart
        setTimeout(function () {
            _this.setState({
                moving: false,
            });
        }, 0);
    };
    // 滑动惯性
    PickerColumn.prototype.momentum = function (distance, duration) {
        var speed = Math.abs(distance / duration);
        distance = this.state.offset + (speed / 0.003) * (distance < 0 ? -1 : 1);
        var index = this.getIndexByOffset(distance);
        this.setState({
            duration: +this.props.swipeDuration,
        });
        this.setIndex(index);
    };
    PickerColumn.prototype.genOptions = function () {
        var _this = this;
        var _a = this.props, options = _a.options, value = _a.value, itemHeight = _a.itemHeight, disabledValue = _a.disabledValue;
        var optionStyle = {
            height: itemHeight + "px",
        };
        return options.map(function (option, index) {
            var data = {
                style: optionStyle,
                className: clsx_1.default(bem('item', {
                    disabled: option.value === disabledValue,
                    selected: option.value === value,
                })),
                onClick: function () {
                    _this.onClickItem(index);
                },
            };
            var childData = {
                className: 'pant-ellipsis',
            };
            return (jsx_runtime_1.jsx("li", __assign({}, data, { children: jsx_runtime_1.jsx("div", __assign({}, childData, { children: option.label }), void 0) }), index));
        });
    };
    PickerColumn.prototype.onTransitionEnd = function () {
        this.stopMomentum();
    };
    PickerColumn.prototype.stopMomentum = function () {
        this.setState({
            moving: false,
            duration: 0,
        });
        if (this.transitionEndTrigger) {
            this.transitionEndTrigger();
            this.transitionEndTrigger = null;
        }
    };
    PickerColumn.prototype.render = function () {
        var _a = this.state, offset = _a.offset, baseOffset = _a.baseOffset, duration = _a.duration;
        var wrapperStyle = {
            transform: "translate3d(0, " + (offset + baseOffset) + "px, 0)",
            transitionDuration: duration + "ms",
            transitionProperty: duration ? 'all' : 'none',
        };
        return (jsx_runtime_1.jsx("div", __assign({ className: clsx_1.default(bem(), this.props.className) }, { children: jsx_runtime_1.jsx("ul", __assign({ ref: this.listRef, style: wrapperStyle, className: bem('wrapper') }, { children: this.genOptions() }), void 0) }), void 0));
    };
    return PickerColumn;
}(react_1.default.PureComponent));
exports.PickerColumn = PickerColumn;
