"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PullRefresh = void 0;
var jsx_runtime_1 = require("react/jsx-runtime");
var react_1 = __importDefault(require("react"));
var utils_1 = require("../utils");
var event_1 = require("../utils/event");
var bem_1 = require("../utils/bem");
var scroll_1 = require("../utils/scroll");
var touch_handler_1 = require("../utils/touch-handler");
var loading_1 = require("../loading");
var locale_1 = require("../locale");
require("./index.scss");
var DEFAULT_HEAD_HEIGHT = 50;
var bem = bem_1.createBEM('pant-pull-refresh');
var PullRefresh = /** @class */ (function (_super) {
    __extends(PullRefresh, _super);
    function PullRefresh(props) {
        var _this = _super.call(this, props) || this;
        _this.containerRef = react_1.default.createRef();
        _this.ceiling = false;
        _this.state = {
            status: 'normal',
            distance: 0,
        };
        return _this;
    }
    PullRefresh.prototype.componentDidMount = function () {
        this.scroller = scroll_1.getScroller(this.containerRef.current);
        this.touchHandler = new touch_handler_1.TouchHandler(this.containerRef.current, {
            onBeforeTouchStart: this.onBeforeTouchStart.bind(this),
            onTouchMove: this.onTouchMove.bind(this),
            onBeforeTouchMove: this.onBeforeTouchMove.bind(this),
            onTouchEnd: this.onTouchEnd.bind(this),
        });
    };
    PullRefresh.prototype.componentWillUnmount = function () {
        this.touchHandler.destroy();
        this.touchHandler = null;
        this.scroller = null;
    };
    Object.defineProperty(PullRefresh.prototype, "touchable", {
        get: function () {
            var status = this.state.status;
            return status !== 'loading' && status !== 'success' && !this.props.disabled;
        },
        enumerable: false,
        configurable: true
    });
    PullRefresh.prototype.checkPullStart = function (event) {
        this.ceiling = scroll_1.getScrollTop(this.scroller) === 0;
        if (this.ceiling) {
            this.touchHandler.touchStart(event);
        }
    };
    PullRefresh.prototype.onBeforeTouchStart = function (event) {
        if (!this.touchable) {
            return false;
        }
        this.checkPullStart(event);
        return false;
    };
    PullRefresh.prototype.onBeforeTouchMove = function (event) {
        if (!this.touchable) {
            return false;
        }
        if (!this.ceiling) {
            this.checkPullStart(event);
        }
        return this.ceiling;
    };
    PullRefresh.prototype.onTouchMove = function (event) {
        var _a = this.touchHandler.state, deltaY = _a.deltaY, direction = _a.direction;
        if (deltaY >= 0 && direction === 'vertical') {
            event_1.preventDefault(event);
            this.setStatus(this.ease(deltaY));
        }
    };
    PullRefresh.prototype.onTouchEnd = function () {
        var _this = this;
        var status = this.state.status;
        var props = this.props;
        var headHeight = props.headHeight, onRefresh = props.onRefresh, successDuration = props.successDuration;
        var deltaY = this.touchHandler.state.deltaY;
        if (this.touchable && this.ceiling && deltaY) {
            if (status === 'loosing' && onRefresh) {
                this.setStatus(+headHeight, true);
                onRefresh().finally(function () {
                    if (props.successNode || props.successText) {
                        _this.setState({ status: 'success' }, function () {
                            setTimeout(function () {
                                _this.setStatus(0);
                            }, +successDuration);
                        });
                    }
                    else {
                        _this.setStatus(0);
                    }
                });
            }
            else {
                this.setStatus(0);
            }
        }
    };
    PullRefresh.prototype.ease = function (distance) {
        var headHeight = +this.props.headHeight;
        if (distance > headHeight) {
            if (distance < headHeight * 2) {
                distance = headHeight + (distance - headHeight) / 2;
            }
            else {
                distance = headHeight * 1.5 + (distance - headHeight * 2) / 4;
            }
        }
        return Math.round(distance);
    };
    PullRefresh.prototype.setStatus = function (distance, isLoading) {
        var status;
        if (isLoading) {
            this.ceiling = false;
            status = 'loading';
        }
        else if (distance === 0) {
            this.ceiling = false;
            status = 'normal';
        }
        else {
            status = distance < this.props.headHeight ? 'pulling' : 'loosing';
        }
        this.setState({ status: status, distance: distance });
    };
    PullRefresh.prototype.genStatus = function () {
        var props = this.props;
        var _a = this.state, status = _a.status, distance = _a.distance;
        if (status === 'success') {
            return props.successNode || jsx_runtime_1.jsx("div", __assign({ className: bem('text') }, { children: props.successText }), void 0);
        }
        if (status === 'loading') {
            return props.loadingNode || jsx_runtime_1.jsx(loading_1.Loading, __assign({ size: "16" }, { children: props.loadingText || locale_1.i18n().loading }), void 0);
        }
        if (status === 'loosing') {
            return props.loosingNode || jsx_runtime_1.jsx("div", __assign({ className: bem('text') }, { children: props.loosingText || locale_1.i18n().pullRefresh.loosing }), void 0);
        }
        if (status === 'pulling') {
            return ((props.pullingNode && react_1.default.cloneElement(props.pullingNode, { distance: distance })) || (jsx_runtime_1.jsx("div", __assign({ className: bem('text') }, { children: props.pullingText || locale_1.i18n().pullRefresh.pulling }), void 0)));
        }
    };
    PullRefresh.prototype.render = function () {
        var _a = this.props, animationDuration = _a.animationDuration, headHeight = _a.headHeight, children = _a.children;
        var distance = this.state.distance;
        var duration = this.ceiling ? 0 : animationDuration;
        var trackStyle = {
            transitionDuration: duration + "ms",
            transform: distance ? "translate3d(0," + distance + "px, 0)" : '',
        };
        var headStyle = headHeight !== DEFAULT_HEAD_HEIGHT
            ? {
                height: utils_1.addUnit(headHeight),
            }
            : null;
        return (jsx_runtime_1.jsx("div", __assign({ ref: this.containerRef, className: bem() }, { children: jsx_runtime_1.jsxs("div", __assign({ className: bem('track'), style: trackStyle }, { children: [jsx_runtime_1.jsx("div", __assign({ className: bem('head'), style: headStyle }, { children: this.genStatus() }), void 0), children] }), void 0) }), void 0));
    };
    PullRefresh.defaultProps = {
        successDuration: 500,
        animationDuration: 300,
        headHeight: DEFAULT_HEAD_HEIGHT,
    };
    return PullRefresh;
}(react_1.default.PureComponent));
exports.PullRefresh = PullRefresh;
