import React from 'react';
import { ListRowProps } from 'react-virtualized';
import './index.scss';
export declare type DataItem = string | Record<string, any>;
export declare type DataSet = DataItem[];
export declare type LoadParams = {
    text: string;
    offset: number;
    limit: number;
    prevData: DataSet;
    loadMore?: boolean;
};
export declare type LoadResult = {
    data: DataSet;
    total: number;
    params: LoadParams;
};
export declare type SearchablePickerProps = {
    title?: string;
    height?: number | string;
    rowHeight?: number;
    maxSelection?: number;
    maxSelectionMsg?: string;
    firstLoadLimit?: number;
    searchable?: boolean;
    fullscreen?: boolean;
    showToolbar?: boolean;
    toolbarPosition?: 'top' | 'bottom';
    cancelButtonText?: string;
    confirmButtonText?: string;
    valueKey?: string | string[];
    labelKey?: string | string[];
    data?: DataSet;
    noDataMsg?: string;
    defaultValue?: string | string[];
    checkedIcon?: JSX.Element;
    uncheckedIcon?: JSX.Element;
    onCancel?: () => void;
    onConfirm?: (value: string[] | string) => void;
    closePopup?: (confirm?: boolean) => void;
    onChange?: (value: string[] | string) => void;
    onLoad?: (params: LoadParams) => Promise<LoadResult>;
    onSelectionExceeds?: () => void;
    rowRenderer?: (rowProps: ListRowProps & {
        selected: boolean;
        item: Record<string, any>;
        select(index: number): void;
    }) => JSX.Element;
    _popupId?: number;
};
declare type SearchablePickerState = {
    pickerValue: string[];
    rollbackPickerValue: string[];
    contentWidth: number;
    contentHeight: number;
    searchText: string;
    loading: boolean;
    data: DataSet;
    total: number;
    prevProps: SearchablePickerProps;
};
export declare class SearchablePicker extends React.PureComponent<SearchablePickerProps, SearchablePickerState> {
    static defaultProps: {
        height: number;
        rowHeight: number;
        maxSelection: number;
        firstLoadLimit: number;
        searchable: boolean;
        showToolbar: boolean;
        toolbarPosition: string;
        valueKey: string;
        labelKey: string;
        data: DataSet;
    };
    private contentRef;
    private listInstance;
    constructor(props: SearchablePickerProps);
    static getDerivedStateFromProps(props: SearchablePickerProps, state: SearchablePickerState): SearchablePickerState;
    componentDidMount(): void;
    componentWillUnmount(): void;
    getValue(): string[] | string;
    clearValue(cb: () => void): void;
    getDataList(): DataSet;
    setLoadResult(res: LoadResult): void;
    showLoading(): void;
    hideLoading(): void;
    onSearchChange(text: string): void;
    onPopupOpened(id: number): void;
    genToolbar(top?: boolean): JSX.Element;
    onConfirm(): void;
    onCancel(): void;
    normalizeItem(item: string | Record<string, any>): Record<string, any>;
    select(index: number): void;
    rowRenderer(rowProps: ListRowProps): JSX.Element;
    isRowLoaded({ index }: {
        index: number;
    }): boolean;
    loadMoreRows({ startIndex, stopIndex }: {
        startIndex: number;
        stopIndex: number;
    }): Promise<any>;
    render(): JSX.Element;
}
export {};
