"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Sticky = void 0;
var jsx_runtime_1 = require("react/jsx-runtime");
var react_1 = __importDefault(require("react"));
var utils_1 = require("../utils");
var dom_1 = require("../utils/dom");
var event_1 = require("../utils/event");
var scroll_1 = require("../utils/scroll");
var bem_1 = require("../utils/bem");
require("./index.scss");
var bem = bem_1.createBEM('pant-sticky');
var Sticky = /** @class */ (function (_super) {
    __extends(Sticky, _super);
    function Sticky() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.domRef = react_1.default.createRef();
        _this.bindedOnScroll = _this.onScroll.bind(_this);
        _this.bindedOnRootScroll = _this.onRootScroll.bind(_this);
        _this.toRef = null;
        _this.state = {
            fixed: false,
            height: 0,
            left: 0,
            width: 0,
            transform: 0,
        };
        return _this;
    }
    Sticky.prototype.componentDidMount = function () {
        var _this = this;
        var el = this.domRef.current;
        this.scroller = this.props.scroller || scroll_1.getScroller(el);
        event_1.on(this.scroller, 'scroll', this.bindedOnScroll);
        if (!scroll_1.isRootScroller(this.scroller)) {
            event_1.on(window, 'scroll', this.bindedOnRootScroll);
        }
        this.onRootScroll();
        this.toRef = setInterval(function () {
            _this.onRootScroll();
        }, 1000);
    };
    Sticky.prototype.componentWillUnmount = function () {
        event_1.off(this.scroller, 'scroll', this.bindedOnScroll);
        if (!scroll_1.isRootScroller(this.scroller)) {
            event_1.off(window, 'scroll', this.bindedOnRootScroll);
        }
    };
    Sticky.prototype.getStyle = function () {
        var _a = this.props, keepWidth = _a.keepWidth, zIndex = _a.zIndex, offsetTop = _a.offsetTop, offsetBottom = _a.offsetBottom, stickBottom = _a.stickBottom;
        var _b = this.state, left = _b.left, width = _b.width, fixed = _b.fixed, transform = _b.transform;
        if (!fixed) {
            return;
        }
        var style = {};
        if (utils_1.isDef(zIndex)) {
            style.zIndex = zIndex;
        }
        if (fixed) {
            if (stickBottom) {
                if (scroll_1.isRootScroller(this.scroller)) {
                    style.bottom = utils_1.addUnit(offsetBottom);
                }
                else {
                    var height = scroll_1.getViewportSize().height;
                    var bottom = this.scroller.getBoundingClientRect().bottom;
                    style.bottom = utils_1.addUnit(+offsetBottom + height - bottom);
                }
            }
            else {
                if (scroll_1.isRootScroller(this.scroller)) {
                    style.top = utils_1.addUnit(offsetTop);
                }
                else {
                    var top_1 = this.scroller.getBoundingClientRect().top;
                    style.top = utils_1.addUnit(+offsetTop + top_1);
                }
            }
        }
        if (keepWidth) {
            style.left = utils_1.addUnit(left);
            style.width = utils_1.addUnit(width);
        }
        if (transform) {
            style.transform = "translate3d(0, " + transform + "px, 0)";
        }
        return style;
    };
    Object.defineProperty(Sticky.prototype, "fixed", {
        get: function () {
            return this.state.fixed;
        },
        enumerable: false,
        configurable: true
    });
    Sticky.prototype.stick = function (isRootScroll) {
        var el = this.domRef.current;
        var _a = el.getBoundingClientRect(), left = _a.left, width = _a.width;
        var height = el.offsetHeight;
        var fixed = false;
        var transform = 0;
        var _b = this.props, container = _b.container, offsetTop = _b.offsetTop, offsetBottom = _b.offsetBottom;
        var offsetTopNumber = utils_1.removeUnit(offsetTop);
        var scrollTop = scroll_1.getScrollTop(this.scroller);
        var topToScrollerTop = scroll_1.getElementTop(el, this.scroller);
        // The sticky component should be kept inside the container element
        if (container && container.current) {
            var containerEl = container.current;
            var bottomToScrollerTop = scroll_1.getElementTop(containerEl, this.scroller) + containerEl.offsetHeight;
            var offsetBottomNumber = utils_1.removeUnit(offsetBottom);
            if (scrollTop + offsetTopNumber + height + offsetBottomNumber > bottomToScrollerTop) {
                var distanceToBottom = height + scrollTop - bottomToScrollerTop;
                if (distanceToBottom < height) {
                    fixed = true;
                    transform = -(distanceToBottom + offsetTopNumber + offsetBottomNumber);
                }
                this.setState({ height: height, left: left, width: width, fixed: fixed, transform: transform });
                if (isRootScroll && !scroll_1.isRootScroller(this.scroller)) {
                    this.forceUpdate();
                }
                return;
            }
        }
        if (scrollTop + offsetTopNumber > topToScrollerTop) {
            fixed = true;
        }
        this.setState({ height: height, left: left, width: width, fixed: fixed, transform: transform });
        if (isRootScroll && !scroll_1.isRootScroller(this.scroller)) {
            this.forceUpdate();
        }
    };
    Sticky.prototype.stickBottom = function (isRootScroll) {
        var el = this.domRef.current;
        var _a = el.getBoundingClientRect(), left = _a.left, width = _a.width;
        var height = el.offsetHeight;
        var fixed = false;
        var transform = 0;
        var _b = this.props, container = _b.container, offsetTop = _b.offsetTop, offsetBottom = _b.offsetBottom;
        var offsetBottomNumber = utils_1.removeUnit(offsetBottom);
        var scrollTop = scroll_1.getScrollTop(this.scroller);
        var topToScrollerTop = scroll_1.getElementTop(el, this.scroller);
        var viewportHeight = scroll_1.getVisibleHeight(this.scroller);
        // The sticky component should be kept inside the container element
        if (container && container.current) {
            var containerEl = container.current;
            var containerTopToScrollerTop = scroll_1.getElementTop(containerEl, this.scroller);
            var offsetTopNumber = utils_1.removeUnit(offsetTop);
            if (containerTopToScrollerTop + offsetTopNumber + height + offsetBottomNumber > scrollTop + viewportHeight) {
                var distanceToBottom = height + containerTopToScrollerTop - scrollTop - viewportHeight;
                if (distanceToBottom < height) {
                    fixed = true;
                    transform = distanceToBottom + offsetTopNumber + offsetBottomNumber;
                }
                this.setState({ height: height, left: left, width: width, fixed: fixed, transform: transform });
                if (isRootScroll && !scroll_1.isRootScroller(this.scroller)) {
                    this.forceUpdate();
                }
                return;
            }
        }
        if (scrollTop + viewportHeight < topToScrollerTop + height + offsetBottomNumber) {
            fixed = true;
        }
        this.setState({ height: height, left: left, width: width, fixed: fixed, transform: transform });
        if (isRootScroll && !scroll_1.isRootScroller(this.scroller)) {
            this.forceUpdate();
        }
    };
    Sticky.prototype.onScroll = function (_, isRootScroll) {
        clearInterval(this.toRef);
        var el = this.domRef.current;
        if (dom_1.isHidden(el)) {
            return;
        }
        var container = this.props.container;
        if (container && !container.current) {
            return;
        }
        this.props.stickBottom ? this.stickBottom(isRootScroll) : this.stick(isRootScroll);
    };
    Sticky.prototype.onRootScroll = function (evt) {
        this.onScroll(evt, true);
    };
    Sticky.prototype.render = function () {
        var props = this.props;
        var _a = this.state, fixed = _a.fixed, height = _a.height;
        var warpperStyle = {
            height: fixed ? height + "px" : null,
        };
        return (jsx_runtime_1.jsx("div", __assign({ ref: this.domRef, style: warpperStyle }, { children: jsx_runtime_1.jsx("div", __assign({ className: bem({ fixed: fixed }), style: this.getStyle() }, { children: props.children }), void 0) }), void 0));
    };
    Sticky.defaultProps = {
        keepWidth: true,
        offsetTop: 0,
        offsetBottom: 0,
    };
    return Sticky;
}(react_1.default.PureComponent));
exports.Sticky = Sticky;
