"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.removeClass = exports.addClass = exports.hasClass = exports.closest = exports.isHidden = exports.removeNode = void 0;
var closest_1 = __importDefault(require("closest"));
function removeNode(el) {
    var parent = el.parentNode;
    if (parent) {
        parent.removeChild(el);
    }
}
exports.removeNode = removeNode;
function isHidden(el) {
    var style = window.getComputedStyle(el);
    var hidden = style.display === 'none';
    // offsetParent returns null in the following situations:
    // 1. The element or its parent element has the display property set to none.
    // 2. The element has the position property set to fixed
    var parentHidden = el.offsetParent === null && style.position !== 'fixed';
    return hidden || parentHidden;
}
exports.isHidden = isHidden;
function closest(element, selector, checkYoSelf) {
    return closest_1.default(element, selector, checkYoSelf);
}
exports.closest = closest;
function hasClass(el, className) {
    return el.className.split(/\s+/).some(function (c) { return c === className; });
}
exports.hasClass = hasClass;
function addClass(el, className) {
    if (hasClass(el, className)) {
        return;
    }
    var oldClassName = el.className;
    el.className = oldClassName ? oldClassName + ' ' + className : className;
}
exports.addClass = addClass;
function removeClass(el, className) {
    var has = false;
    var cs = el.className.split(/\s+/).filter(function (c) {
        if (c === className) {
            has = true;
            return false;
        }
        return true;
    });
    if (has) {
        el.className = cs.join(' ');
    }
}
exports.removeClass = removeClass;
