"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.interpolate = exports.getIncrementalZIndex = exports.omit = exports.removeUnit = exports.addUnit = exports.isNaN = exports.isNumeric = exports.isObject = exports.isDef = exports.inBrowser = void 0;
var __1 = require("../");
var constant_1 = require("./constant");
exports.inBrowser = typeof window !== 'undefined';
function isDef(val) {
    return val !== undefined && val !== null;
}
exports.isDef = isDef;
function isObject(val) {
    return val !== null && typeof val === 'object';
}
exports.isObject = isObject;
function isNumeric(val) {
    return /^\d+(\.\d+)?$/.test(val);
}
exports.isNumeric = isNumeric;
function isNaN(val) {
    if (Number.isNaN) {
        return Number.isNaN(val);
    }
    return val !== val;
}
exports.isNaN = isNaN;
function addUnit(value) {
    if (!isDef(value)) {
        return undefined;
    }
    var viewportWidth = __1.pantConfig('viewportWidth');
    value = value + '';
    if (isNumeric(value)) {
        if (viewportWidth > 0) {
            return (+value * 100) / viewportWidth + 'vw';
        }
        else {
            return value + 'px';
        }
    }
    else {
        return value;
    }
}
exports.addUnit = addUnit;
function removeUnit(value) {
    if (!isDef(value)) {
        return undefined;
    }
    if (typeof value === 'number') {
        return value;
    }
    else {
        var n = parseFloat(value);
        if (exports.inBrowser && /vw$/.test(value)) {
            return (screen.availWidth * n) / 100;
        }
        else {
            return n;
        }
    }
}
exports.removeUnit = removeUnit;
function omit(obj, keys) {
    var res = {};
    Object.keys(obj)
        .filter(function (key) {
        return !keys.includes(key);
    })
        .forEach(function (key) {
        res[key] = obj[key];
    });
    return res;
}
exports.omit = omit;
var zIndex = constant_1.Z_INDEX_INCREMENTAL_START;
function getIncrementalZIndex(base) {
    if (base === void 0) { base = 0; }
    return base + zIndex++;
}
exports.getIncrementalZIndex = getIncrementalZIndex;
function interpolate(text, data) {
    return (text &&
        text.replace(/\{(.*?)\}/g, function (full, key) {
            var res = data && data[key];
            return isDef(res) ? res : full;
        }));
}
exports.interpolate = interpolate;
