"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.scrollTopTo = exports.scrollLeftTo = exports.getViewportSize = exports.getVisibleBottom = exports.getVisibleTop = exports.getVisibleHeight = exports.getElementTop = exports.setRootScrollTop = exports.getRootScrollTop = exports.setScrollTop = exports.getScrollTop = exports.isRootScroller = exports.getScroller = void 0;
function isWindow(val) {
    return val === window;
}
// get nearest scroll element
// http://w3help.org/zh-cn/causes/SD9013
// http://stackoverflow.com/questions/17016740/onscroll-function-is-not-working-for-chrome
var overflowScrollReg = /scroll|auto/i;
function getScroller(el, root) {
    if (root === void 0) { root = window; }
    var node = el;
    while (node && node.tagName !== 'HTML' && node.nodeType === 1 && node !== root) {
        var overflowY = getComputedStyle(node).overflowY;
        if (overflowScrollReg.test(overflowY)) {
            if (node.tagName !== 'BODY') {
                return node;
            }
            // see: https://github.com/youzan/vant/issues/3823
            var htmlOverflowY = getComputedStyle(node.parentNode).overflowY;
            if (overflowScrollReg.test(htmlOverflowY)) {
                return node;
            }
        }
        node = node.parentNode;
    }
    return root;
}
exports.getScroller = getScroller;
function isRootScroller(el) {
    if (el === window) {
        return true;
    }
    var tagName = el.tagName;
    return tagName === 'BODY' || tagName === 'HTML';
}
exports.isRootScroller = isRootScroller;
function getScrollTop(el) {
    if (isRootScroller(el)) {
        return getRootScrollTop();
    }
    return 'scrollTop' in el ? el.scrollTop : el.pageYOffset;
}
exports.getScrollTop = getScrollTop;
function setScrollTop(el, value) {
    if ('scrollTop' in el) {
        el.scrollTop = value;
    }
    else {
        el.scrollTo(el.scrollX, value);
    }
}
exports.setScrollTop = setScrollTop;
function getRootScrollTop() {
    return window.pageYOffset || document.documentElement.scrollTop || document.body.scrollTop || 0;
}
exports.getRootScrollTop = getRootScrollTop;
function setRootScrollTop(value) {
    setScrollTop(window, value);
    setScrollTop(document.body, value);
}
exports.setRootScrollTop = setRootScrollTop;
// get distance from element top to scroller content top
function getElementTop(el, scroller) {
    if (isWindow(el)) {
        return 0;
    }
    if (!scroller || isRootScroller(scroller)) {
        return el.getBoundingClientRect().top + getRootScrollTop();
    }
    return (el.getBoundingClientRect().top - scroller.getBoundingClientRect().top + getScrollTop(scroller));
}
exports.getElementTop = getElementTop;
function getVisibleHeight(el) {
    if (isWindow(el)) {
        return el.innerHeight;
    }
    return el.getBoundingClientRect().height;
}
exports.getVisibleHeight = getVisibleHeight;
function getVisibleTop(el) {
    if (isWindow(el)) {
        return 0;
    }
    return el.getBoundingClientRect().top;
}
exports.getVisibleTop = getVisibleTop;
function getVisibleBottom(el) {
    if (isWindow(el)) {
        return 0;
    }
    return el.getBoundingClientRect().bottom;
}
exports.getVisibleBottom = getVisibleBottom;
function getViewportSize() {
    return {
        width: Math.max(document.documentElement.clientWidth || 0, window.innerWidth || 0),
        height: Math.max(document.documentElement.clientHeight || 0, window.innerHeight || 0),
    };
}
exports.getViewportSize = getViewportSize;
function scrollLeftTo(scroller, to, duration) {
    var count = 0;
    var from = scroller.scrollLeft;
    var frames = duration === 0 ? 1 : Math.round((duration * 1000) / 16);
    var step = (to - from) / frames;
    function animate() {
        scroller.scrollLeft += step;
        if (++count < frames) {
            requestAnimationFrame(animate);
        }
    }
    animate();
}
exports.scrollLeftTo = scrollLeftTo;
function scrollTopTo(scroller, to, duration, callback) {
    var current = getScrollTop(scroller);
    var isDown = current < to;
    var frames = duration === 0 ? 1 : Math.round((duration * 1000) / 16);
    var step = (to - current) / frames;
    function animate() {
        current += step;
        if ((isDown && current > to) || (!isDown && current < to)) {
            current = to;
        }
        setScrollTop(scroller, current);
        if ((isDown && current < to) || (!isDown && current > to)) {
            requestAnimationFrame(animate);
        }
        else if (callback) {
            requestAnimationFrame(callback);
        }
    }
    animate();
}
exports.scrollTopTo = scrollTopTo;
